#!/usr/bin/perl
BEGIN{

    use strict;
    
    my @needed_libs = (
        'Spreadsheet::WriteExcel', 
        'Spreadsheet::WriteExcel::Format', 
        'Spreadsheet::WriteExcel::Utility',  
        );
    
    # Check if this method can be used   
    foreach (@needed_libs)
    {
    	eval "require $_";
	    if ($@)
	    {
            print ("Unable to find libraries $_\n");
	    	exit(1);
	    };
    }
}

use strict;
use warnings;
use Spreadsheet::WriteExcel;
use Spreadsheet::WriteExcel::Format;
use Spreadsheet::WriteExcel::Utility;
use Data::Dumper;


my $module_name=$0;
my $module_path=$0;

$module_name=~s/([^\/\\]+)$//;
$module_name=$1;
 
$module_path=~s/([^\/\\]+)$//;           

use Getopt::Long qw(:config no_ignore_case);

my $opt_help;
my @opt_src;
my $opt_dest;
my $opt_title;
my $opt_mailto;

my $opt_tab_title = 1;
my $opt_tab_view = 1;
my $opt_tab_data = 1;
my $opt_chart_on = 1;


my $report_file = ();
my $report_title = ();
my @files = ();

GetOptions ("help|h"=>\$opt_help, "src|s=s" => \@opt_src, "dest|o=s" => \$opt_dest, "title|t=s" => \$opt_title, "email|m|e=s" => \$opt_mailto);

if ( !@opt_src || $opt_help ) 
{
    help();
    exit;
}

$report_file = $opt_dest ? $opt_dest : "osudiff_report.xls";
$report_title = $opt_title ? $opt_title : "List of objects to compare";

if ($#opt_src != (-1))
{
    @files=split(/,/,join(',',@opt_src));
}

verify_opt_file( @files );

my %report_conf = ('title' => $report_title,  'dest' => $report_file, 'src' => \@files, );
create_report(\%report_conf);
if ( $opt_mailto ) 
{
    send_results_by_mail($opt_mailto, $report_file);
}

if(-e $report_file)
{
    print ("Report file $report_file is ready\n");
}

exit 0;

sub help 
{
    print ("Usage: $module_name [OPTION...]\n");
    print ("\'$module_name\' generates excel file, comparing imb output from two diff jobs.\n\n");

    print ("\nOptions:\n");
    printf (" %-5s %-10s\t%-s\n", '-h,', '--help', "Show the help message and exit.");
    printf (" %-5s %-10s\t%-s\n", '-s,', '--src=[FILE]', "Input file.");
    printf (" %-5s %-10s\t%-s\n", '-o,', '--dest=[FILE]', "Output Excel file name (default is 'report.xls').");
}

sub verify_opt_file
{
    my (@files)=@_;
    foreach my $file (@files) 
    {
        if( ! -e $file)
        {
            die "$file doesn't exist";
        }
    }
}

sub create_report 
{
	my $workbook = Spreadsheet::WriteExcel->new($report_file);
	my $worksheet = $workbook->add_worksheet('result');
	
	my $bw_graphs;
	my $bw_graphs_chart;
    my $lat_graphs;
	my $lat_graphs_chart;
	my $pps_graphs;
	my $pps_graphs_chart;

	my $s_bw_graphs;
	my $s_bw_graphs_chart;

	my $s_bibw_graphs;
	my $s_bibw_graphs_chart;

	my ($report_conf)=@_;
	my $ret={};
    foreach my $file (@{$report_conf->{src}}) 
    {
        open(FILE, "< $file") ||  die "Can't open $file: $!\n" if defined($file);
        read FILE, my $output, -s FILE;
		analyze_osu($output,$file,$ret,$workbook);
        close(FILE);      
    }
	#print "gresult ",Dumper($ret),"\n";	
	
	my $file_name  = $workbook->add_format( merge=>1,align=>'vcenter',size=>11);
	my @file_name = ("HPC Verification Status", '','','','','','');
	my $vert_pos_lat = "B";
	my $vert_pos_mbw_mr = "B";
	my $vert_pos_bw = "B";
	my $vert_pos_bibw = "B";

	foreach my $kfile_name (keys(%{$ret}))
	{
		my $horz_pos_lat=32;
		my $horz_pos_mbw_mr=3;
		my $horz_pos_bw=60;
		my $horz_pos_bibw=90;
		if($ret->{$kfile_name}->{"name"} eq "osu_latency")
		{
			if(!defined($lat_graphs))
			{
				$lat_graphs = $workbook->add_worksheet('lat_graphs');
				$lat_graphs_chart = $workbook->add_chart( type => 'line' , embedded => 1);
				$lat_graphs_chart->set_title( name => 'OSU MPI Latency' );
				$lat_graphs_chart->set_x_axis( name => 'Size' );
				$lat_graphs_chart->set_y_axis( name => 'Latency (us)' );
			}
			$worksheet->write_row("$vert_pos_lat"."$horz_pos_lat", [$kfile_name,'','','',''], $file_name);
			$horz_pos_lat++;
			$worksheet->write("$vert_pos_lat"."$horz_pos_lat","PPN");
			$vert_pos_lat++;
			
			$worksheet->write("$vert_pos_lat"."$horz_pos_lat","MSGSIZE");
			$worksheet->set_column("$vert_pos_lat:"."$vert_pos_lat", 11);
			foreach (@{$ret->{$kfile_name}->{"values"}->{0}})
			{
				$horz_pos_lat++;
				$worksheet->write("$vert_pos_lat"."$horz_pos_lat",$_);
			}
			my $l_vert_pos_lat = $vert_pos_lat;
			my $l_horz_pos_lat = $horz_pos_lat;
			$vert_pos_lat++;
			$horz_pos_lat=33;
			
			$worksheet->write("$vert_pos_lat"."$horz_pos_lat","Latency (us)");
			$worksheet->set_column("$vert_pos_lat:"."$vert_pos_lat", 11);
			foreach (@{$ret->{$kfile_name}->{"values"}->{1}})
			{
				$horz_pos_lat++;
				$worksheet->write("$vert_pos_lat"."$horz_pos_lat",$_);
			}
			
			$lat_graphs_chart->add_series(categories=> '=result!$' . "$l_vert_pos_lat" . '$34:$' . "$l_vert_pos_lat" . '$' . "$l_horz_pos_lat",
											values=>'=result!$' . "$vert_pos_lat" . '$34:$' . "$vert_pos_lat" . '$' . "$horz_pos_lat" ,
											name=> $kfile_name);
			$lat_graphs->insert_chart('B2', $lat_graphs_chart,100,100, 1.5, 3.0);
			$vert_pos_lat++;
			$vert_pos_lat++;
			$vert_pos_lat++;
		} elsif($ret->{$kfile_name}->{"name"} eq "osu_mbw_mr")
		{
			if(!defined($bw_graphs))
			{
				$bw_graphs = $workbook->add_worksheet('bw_mr_graphs');
				$bw_graphs_chart = $workbook->add_chart( type => 'line' , embedded => 1);
				$bw_graphs_chart->set_title( name => 'OSU MPI Multiple Bandwidth' );
				$bw_graphs_chart->set_x_axis( name => 'Size' );
				$bw_graphs_chart->set_y_axis( name => 'MB/s' );
		
				$pps_graphs = $workbook->add_worksheet('pps_graphs');
				$pps_graphs_chart = $workbook->add_chart( type => 'line' , embedded => 1);
				$pps_graphs_chart->set_title( name => 'OSU MPI Message Rate' );
				$pps_graphs_chart->set_x_axis( name => 'Size' );
				$pps_graphs_chart->set_y_axis( name => 'Messages/s' );
			}			
			$worksheet->write_row("$vert_pos_mbw_mr"."$horz_pos_mbw_mr", [$kfile_name,'','','',''], $file_name);
			$horz_pos_mbw_mr++;
			$worksheet->write("$vert_pos_mbw_mr"."$horz_pos_mbw_mr","PPN");
			$vert_pos_mbw_mr++;
			
			$worksheet->write("$vert_pos_mbw_mr"."$horz_pos_mbw_mr","MSGSIZE");
			$worksheet->set_column("$vert_pos_mbw_mr:"."$vert_pos_mbw_mr", 11);
			foreach (@{$ret->{$kfile_name}->{"values"}->{0}})
			{
				$horz_pos_mbw_mr++;
				$worksheet->write("$vert_pos_mbw_mr"."$horz_pos_mbw_mr",$_);
			}
			my $l_vert_pos_mbw_mr = $vert_pos_mbw_mr;
			my $l_horz_pos_mbw_mr = $horz_pos_mbw_mr;
			$vert_pos_mbw_mr++;
			$horz_pos_mbw_mr=4;
			
			#print "$vert_pos_mbw_mr $horz_pos_mbw_mr\n";

			$worksheet->write("$vert_pos_mbw_mr"."$horz_pos_mbw_mr",'MB/S');
			$worksheet->set_column("$vert_pos_mbw_mr:"."$vert_pos_mbw_mr", 11);
			foreach (@{$ret->{$kfile_name}->{"values"}->{1}})
			{
				$horz_pos_mbw_mr++;
				$worksheet->write("$vert_pos_mbw_mr"."$horz_pos_mbw_mr",$_);
			}
			
			$bw_graphs_chart->add_series(categories=> '=result!$' . "$l_vert_pos_mbw_mr" . '$5:$' . "$l_vert_pos_mbw_mr" . '$' . "$l_horz_pos_mbw_mr",
											values=>'=result!$' . "$vert_pos_mbw_mr" . '$5:$' . "$vert_pos_mbw_mr" . '$' . "$horz_pos_mbw_mr",
											name=> $kfile_name);
			$bw_graphs->insert_chart('B2', $bw_graphs_chart);
			$vert_pos_mbw_mr++;
			$horz_pos_mbw_mr=4;
			
			$worksheet->write("$vert_pos_mbw_mr"."$horz_pos_mbw_mr","PPS");
			$worksheet->set_column("$vert_pos_mbw_mr:"."$vert_pos_mbw_mr", 11);
			foreach (@{$ret->{$kfile_name}->{"values"}->{2}})
			{
				$horz_pos_mbw_mr++;
				$worksheet->write("$vert_pos_mbw_mr"."$horz_pos_mbw_mr",$_);
			}
			$pps_graphs_chart->add_series(categories=> '=result!$' . "$l_vert_pos_mbw_mr" . '$5:$' . "$l_vert_pos_mbw_mr" . '$' . "$l_horz_pos_mbw_mr",
											values=>'=result!$' . "$vert_pos_mbw_mr" . '$5:$' . "$vert_pos_mbw_mr" . '$' . "$horz_pos_mbw_mr",
											name=> $kfile_name);
			$pps_graphs->insert_chart('B2', $pps_graphs_chart);

			$vert_pos_mbw_mr++;
			$vert_pos_mbw_mr++;
		} elsif($ret->{$kfile_name}->{"name"} eq "osu_bibw")
        {
			if(!defined($s_bibw_graphs))
			{
				$s_bibw_graphs = $workbook->add_worksheet('bibw_graphs');
				$s_bibw_graphs_chart = $workbook->add_chart( type => 'line' , embedded => 1);
				$s_bibw_graphs_chart->set_title( name => 'OSU MPI Bi-Directional Multiple Bandwidth' );
				$s_bibw_graphs_chart->set_x_axis( name => 'Size' );
				$s_bibw_graphs_chart->set_y_axis( name => 'MB/s' );
			}
			$worksheet->write_row("$vert_pos_bibw"."$horz_pos_bibw", [$kfile_name,'','','',''], $file_name);
			$horz_pos_bibw++;
			$worksheet->write("$vert_pos_bibw"."$horz_pos_bibw","PPN");
			$vert_pos_bibw++;
			
			$worksheet->write("$vert_pos_bibw"."$horz_pos_bibw","MSGSIZE");
			$worksheet->set_column("$vert_pos_bibw:"."$vert_pos_bibw", 11);
			foreach (@{$ret->{$kfile_name}->{"values"}->{0}})
			{
				$horz_pos_bibw++;
				$worksheet->write("$vert_pos_bibw"."$horz_pos_bibw",$_);
			}
			my $l_vert_pos_bibw = $vert_pos_bibw;
			my $l_horz_pos_bibw = $horz_pos_bibw;
			$vert_pos_bibw++;
			$horz_pos_bibw=91;
			
			$worksheet->write("$vert_pos_bibw"."$horz_pos_bibw","BI-Directional Bandwidth (MB/s)");
			$worksheet->set_column("$vert_pos_bibw:"."$vert_pos_bibw", 11);
			foreach (@{$ret->{$kfile_name}->{"values"}->{1}})
			{
				$horz_pos_bibw++;
				$worksheet->write("$vert_pos_bibw"."$horz_pos_bibw",$_);
			}
			
			$s_bibw_graphs_chart->add_series(categories=> '=result!$' . "$l_vert_pos_bibw" . '$91:$' . "$l_vert_pos_bibw" . '$' . "$l_horz_pos_bibw",
											values=>'=result!$' . "$vert_pos_bibw" . '$91:$' . "$vert_pos_bibw" . '$' . "$horz_pos_bibw" ,
											name=> $kfile_name);
			$s_bibw_graphs->insert_chart('B2', $s_bibw_graphs_chart,100,100, 2.0, 3.0);
			$vert_pos_bibw++;
			$vert_pos_bibw++;
			$vert_pos_bibw++;
		} else 
		{
			if(!defined($s_bw_graphs))
			{
				$s_bw_graphs = $workbook->add_worksheet('bw_graphs');
				$s_bw_graphs_chart = $workbook->add_chart( type => 'line' , embedded => 1);
				$s_bw_graphs_chart->set_title( name => 'OSU MPI Multiple Bandwidth' );
				$s_bw_graphs_chart->set_x_axis( name => 'Size' );
				$s_bw_graphs_chart->set_y_axis( name => 'MB/s' );
			}
			$worksheet->write_row("$vert_pos_bw"."$horz_pos_bw", [$kfile_name,'','','',''], $file_name);
			$horz_pos_bw++;
			$worksheet->write("$vert_pos_bw"."$horz_pos_bw","PPN");
			$vert_pos_bw++;
			
			$worksheet->write("$vert_pos_bw"."$horz_pos_bw","MSGSIZE");
			$worksheet->set_column("$vert_pos_bw:"."$vert_pos_bw", 11);
			foreach (@{$ret->{$kfile_name}->{"values"}->{0}})
			{
				$horz_pos_bw++;
				$worksheet->write("$vert_pos_bw"."$horz_pos_bw",$_);
			}
			my $l_vert_pos_bw = $vert_pos_bw;
			my $l_horz_pos_bw = $horz_pos_bw;
			$vert_pos_bw++;
			$horz_pos_bw=61;
			
			$worksheet->write("$vert_pos_bw"."$horz_pos_bw","Bandwidth (MB/s)");
			$worksheet->set_column("$vert_pos_bw:"."$vert_pos_bw", 11);
			foreach (@{$ret->{$kfile_name}->{"values"}->{1}})
			{
				$horz_pos_bw++;
				$worksheet->write("$vert_pos_bw"."$horz_pos_bw",$_);
			}
			
			$s_bw_graphs_chart->add_series(categories=> '=result!$' . "$l_vert_pos_bw" . '$61:$' . "$l_vert_pos_bw" . '$' . "$l_horz_pos_bw",
											values=>'=result!$' . "$vert_pos_bw" . '$61:$' . "$vert_pos_bw" . '$' . "$horz_pos_bw" ,
											name=> $kfile_name);
			$s_bw_graphs->insert_chart('B2', $s_bw_graphs_chart,100,100, 2.0, 3.0);
			$vert_pos_bw++;
			$vert_pos_bw++;
			$vert_pos_bw++;
		}
	}
}

sub analyze_osu 
{

    my($result_stdout,$file_name,$ret) = @_;

    if ($result_stdout =~ m/Test v([\d\.]+)/) 
	{
		$ret->{$file_name}->{"version"} = $1;
    }else
	{

		$ret->{$file_name}->{"version"} = "unknow";
	}

	if($result_stdout =~ m/Latency/)
	{
		$ret->{$file_name}->{"name"} = "osu_latency";
		
	}elsif($result_stdout =~ m/Multiple Bandwidth \/ Message/)
	{
		$ret->{$file_name}->{"name"} = "osu_mbw_mr";

	}elsif($result_stdout =~ m/OSU MPI Bandwidth Test/)
	{
		$ret->{$file_name}->{"name"} = "osu_bw";

	}elsif($result_stdout =~ m/OSU MPI Bi-Directional Bandwidth/)
	{
        $ret->{$file_name}->{"name"} = "osu_bibw";
	}
	else
	{
		die "unknow test: $file_name\n";
	}
	
    foreach (split(/[\n|\r]/i, $result_stdout))
    {
		if($_ =~ m/^\d+\ +(\d+\.\ *){1,2}/)	
		{
			$_ =~ s/\ +/\ /g;
			my @t_array = split(/\ /,$_);
			push(@{$ret->{$file_name}->{"values"}->{0}},$t_array[0]);
			push(@{$ret->{$file_name}->{"values"}->{1}},$t_array[1]);
			push(@{$ret->{$file_name}->{"values"}->{2}},$t_array[2]);
#			print "$_\n";
		}
	}
	
}

